"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServer = void 0;
const cron_handler_1 = require("./cron-handler");
const body_parser_1 = __importDefault(require("body-parser"));
const crypto_1 = require("crypto");
const express_1 = __importDefault(require("express"));
const http_1 = __importDefault(require("http"));
const socket_io_1 = require("socket.io");
const flows_endpoint_1 = require("./flows-endpoint");
const guards_1 = require("./guards");
const logger_1 = require("./logger");
const get_module_export_1 = require("./node/get-module-export");
const steps_1 = require("./steps");
const utils_1 = require("./utils");
const createServer = async (lockedData, eventManager, state) => {
    const app = (0, express_1.default)();
    const server = http_1.default.createServer(app);
    const io = new socket_io_1.Server(server);
    const allSteps = [...steps_1.systemSteps, ...lockedData.activeSteps];
    const cronManager = (0, cron_handler_1.setupCronHandlers)(lockedData, eventManager, io);
    const asyncHandler = (step, flows) => {
        return async (req, res) => {
            const traceId = (0, crypto_1.randomUUID)();
            const logger = new logger_1.Logger(traceId, flows, step.config.name, io);
            logger.debug('[API] Received request, processing step', { path: req.path, step });
            const handler = (await (0, get_module_export_1.getModuleExport)(step.filePath, 'handler'));
            const request = {
                body: req.body,
                headers: req.headers,
                pathParams: req.params,
                queryParams: req.query,
            };
            const emit = async ({ data, type }) => {
                if (!(0, utils_1.isAllowedToEmit)(step, type)) {
                    lockedData.printer.printInvalidEmit(step, type);
                    return;
                }
                await eventManager.emit({ data, type, traceId, flows, logger }, step.filePath);
            };
            try {
                const result = await handler(request, { emit, state, logger, traceId });
                if (result.headers) {
                    Object.entries(result.headers).forEach(([key, value]) => res.setHeader(key, value));
                }
                res.status(result.status);
                res.json(result.body);
            }
            catch (error) {
                logger.error('[API] Internal server error', { error });
                console.log(error);
                res.status(500).json({ error: 'Internal server error' });
            }
        };
    };
    app.use(body_parser_1.default.json());
    app.use(body_parser_1.default.urlencoded({ extended: true }));
    const router = express_1.default.Router();
    const addRoute = (step) => {
        const { method, flows, path } = step.config;
        logger_1.globalLogger.debug('[API] Registering route', step.config);
        if (method === 'POST') {
            router.post(path, asyncHandler(step, flows));
        }
        else if (method === 'GET') {
            router.get(path, asyncHandler(step, flows));
        }
        else {
            throw new Error(`Unsupported method: ${method}`);
        }
    };
    const removeRoute = (step) => {
        const { path, method } = step.config;
        const routerStack = router.stack;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const filteredStack = routerStack.filter((layer) => {
            if (layer.route) {
                const match = layer.route.path === path && layer.route.methods[method.toLowerCase()];
                return !match;
            }
            return true;
        });
        router.stack = filteredStack;
    };
    allSteps.filter(guards_1.isApiStep).forEach(addRoute);
    app.use(router);
    (0, flows_endpoint_1.flowsEndpoint)(lockedData, app);
    server.on('error', (error) => {
        console.error('Server error:', error);
    });
    const close = async () => {
        cronManager.close();
        await io.close();
        server.close();
    };
    return { app, server, socketServer: io, close, removeRoute, addRoute, cronManager };
};
exports.createServer = createServer;
