"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateStep = void 0;
const zod_1 = require("zod");
const emits = zod_1.z.array(zod_1.z.union([
    zod_1.z.string(),
    zod_1.z
        .object({
        type: zod_1.z.string(),
        label: zod_1.z.string().optional(),
        conditional: zod_1.z.boolean().optional(),
    })
        .strict(),
]));
const noopSchema = zod_1.z
    .object({
    type: zod_1.z.literal('noop'),
    name: zod_1.z.string(),
    description: zod_1.z.string().optional(),
    virtualEmits: emits,
    virtualSubscribes: zod_1.z.array(zod_1.z.string()),
    flows: zod_1.z.array(zod_1.z.string()).optional(),
})
    .strict();
const eventSchema = zod_1.z
    .object({
    type: zod_1.z.literal('event'),
    name: zod_1.z.string(),
    description: zod_1.z.string().optional(),
    subscribes: zod_1.z.array(zod_1.z.string()),
    emits: emits,
    virtualEmits: emits.optional(),
    input: zod_1.z.any(),
    flows: zod_1.z.array(zod_1.z.string()).optional(),
})
    .strict();
const apiSchema = zod_1.z
    .object({
    type: zod_1.z.literal('api'),
    name: zod_1.z.string(),
    description: zod_1.z.string().optional(),
    path: zod_1.z.string(),
    method: zod_1.z.string(),
    emits: emits,
    virtualEmits: emits.optional(),
    virtualSubscribes: zod_1.z.array(zod_1.z.string()).optional(),
    flows: zod_1.z.array(zod_1.z.string()).optional(),
    bodySchema: zod_1.z.instanceof(zod_1.z.ZodObject).optional(),
})
    .strict();
const cronSchema = zod_1.z
    .object({
    type: zod_1.z.literal('cron'),
    name: zod_1.z.string(),
    description: zod_1.z.string().optional(),
    cron: zod_1.z.string(),
    virtualEmits: emits.optional(),
    emits: emits,
    flows: zod_1.z.array(zod_1.z.string()).optional(),
})
    .strict();
const validateStep = (step) => {
    try {
        if (step.config.type === 'noop') {
            noopSchema.parse(step.config);
        }
        else if (step.config.type === 'event') {
            eventSchema.parse(step.config);
        }
        else if (step.config.type === 'api') {
            apiSchema.parse(step.config);
        }
        else if (step.config.type === 'cron') {
            cronSchema.parse(step.config);
        }
        else {
            return {
                success: false,
                error: 'Invalid step type',
            };
        }
        return { success: true };
    }
    catch (error) {
        if (error instanceof zod_1.z.ZodError) {
            return {
                success: false,
                error: 'Invalid step configuration',
                errors: error.errors.map((err) => ({ path: err.path.join('.'), message: err.message })),
            };
        }
        // Handle unexpected errors
        return {
            success: false,
            error: 'Unexpected validation error occurred',
        };
    }
};
exports.validateStep = validateStep;
